@@ -14,7 +14,7 @@ import numpy as np
 import ipywidgets as widgets
 from traitlets import CBool, CFloat, Unicode, CaselessStrEnum, List, TraitError, validate
 from ipydatawidgets import NDArray, array_serialization, shape_constraints
-from .trait_types import ITKImage, PointSetList, PolyDataList, itkimage_serialization, polydata_list_serialization
+from .trait_types import ITKImage, PointSetList, PolyDataList, itkimage_serialization, polydata_list_serialization, Colormap
 
 try:
     import ipywebrtc
@@ -29,6 +29,13 @@ try:
 except ImportError:
      pass
 
+have_mayavi = False
+try:
+     import mayavi
+     have_mayavi = True
+except ImportError:
+     pass
+
 import matplotlib
 import colorcet
 
@@ -36,88 +43,6 @@ from . import cm
 
 #from IPython.core.debugger import set_trace
 
-
-COLORMAPS = ("2hot",
-    "Asymmtrical Earth Tones (6_21b)",
-    "Black, Blue and White",
-    "Black, Orange and White",
-    "Black-Body Radiation",
-    "Blue to Red Rainbow",
-    "Blue to Yellow",
-    "Blues",
-    "BrBG",
-    "BrOrYl",
-    "BuGn",
-    "BuGnYl",
-    "BuPu",
-    "BuRd",
-    "CIELab Blue to Red",
-    "Cold and Hot",
-    "Cool to Warm",
-    "Cool to Warm (Extended)",
-    "GBBr",
-    "GYPi",
-    "GnBu",
-    "GnBuPu",
-    "GnRP",
-    "GnYlRd",
-    "Grayscale",
-    "Green-Blue Asymmetric Divergent (62Blbc)",
-    "Greens",
-    "GyRd",
-    "Haze",
-    "Haze_cyan",
-    "Haze_green",
-    "Haze_lime",
-    "Inferno (matplotlib)",
-    "Linear Blue (8_31f)",
-    "Linear YGB 1211g",
-    "Magma (matplotlib)",
-    "Muted Blue-Green",
-    "OrPu",
-    "Oranges",
-    "PRGn",
-    "PiYG",
-    "Plasma (matplotlib)",
-    "PuBu",
-    "PuOr",
-    "PuRd",
-    "Purples",
-    "Rainbow Blended Black",
-    "Rainbow Blended Grey",
-    "Rainbow Blended White",
-    "Rainbow Desaturated",
-    "RdOr",
-    "RdOrYl",
-    "RdPu",
-    "Red to Blue Rainbow",
-    "Reds",
-    "Spectral_lowBlue",
-    "Viridis (matplotlib)",
-    "Warm to Cool",
-    "Warm to Cool (Extended)",
-    "X Ray",
-    "Yellow 15",
-    "blot",
-    "blue2cyan",
-    "blue2yellow",
-    "bone_Matlab",
-    "coolwarm",
-    "copper_Matlab",
-    "gist_earth",
-    "gray_Matlab",
-    "heated_object",
-    "hsv",
-    "hue_L60",
-    "jet",
-    "magenta",
-    "nic_CubicL",
-    "nic_CubicYF",
-    "nic_Edge",
-    "pink_Matlab",
-    "rainbow")
-
-
 def get_ioloop():
     import IPython
     import zmq
@@ -183,13 +108,17 @@ class Viewer(ViewerParent):
     _model_name = Unicode('ViewerModel').tag(sync=True)
     _view_module = Unicode('itkwidgets').tag(sync=True)
     _model_module = Unicode('itkwidgets').tag(sync=True)
-    _view_module_version = Unicode('^0.20.1').tag(sync=True)
-    _model_module_version = Unicode('^0.20.1').tag(sync=True)
+    _view_module_version = Unicode('^0.22.0').tag(sync=True)
+    _model_module_version = Unicode('^0.22.0').tag(sync=True)
     image = ITKImage(default_value=None, allow_none=True, help="Image to visualize.").tag(sync=False, **itkimage_serialization)
     rendered_image = ITKImage(default_value=None, allow_none=True).tag(sync=True, **itkimage_serialization)
     _rendering_image = CBool(default_value=False, help="We are currently volume rendering the image.").tag(sync=True)
     interpolation = CBool(default_value=True, help="Use linear interpolation in slicing planes.").tag(sync=True)
-    cmap = Unicode('Viridis (matplotlib)').tag(sync=True)
+    cmap = Colormap('Viridis (matplotlib)').tag(sync=True)
+    _custom_cmap = NDArray(dtype=np.float32, default_value=None, allow_none=True,
+                help="RGB triples from 0.0 to 1.0 that define a custom linear, sequential colormap")\
+            .tag(sync=True, **array_serialization)\
+            .valid(shape_constraints(None, 3))
     shadow = CBool(default_value=True, help="Use shadowing in the volume rendering.").tag(sync=True)
     slicing_planes = CBool(default_value=False, help="Display the slicing planes in volume rendering view mode.").tag(sync=True)
     gradient_opacity = CFloat(default_value=0.2, help="Volume rendering gradient opacity, from (0.0, 1.0]").tag(sync=True)
@@ -405,13 +334,6 @@ class Viewer(ViewerParent):
             return 1.0
         return value
 
-    @validate('cmap')
-    def _validate_cmap(self, proposal):
-        value = proposal['value']
-        if not value in COLORMAPS:
-            raise TraitError('Invalid colormap')
-        return value
-
     @validate('point_set_colors')
     def _validate_point_set_colors(self, proposal):
         value = proposal['value']
@@ -681,6 +603,12 @@ def view(image=None,
         images = []
 
         for a in actors:
+            if have_mayavi:
+                from tvtk.api import tvtk
+                if isinstance(a, mayavi.modules.surface.Surface):
+                    a = tvtk.to_vtk(a.actor.actor)
+                elif isinstance(a, mayavi.modules.iso_surface.IsoSurface):
+                    a = tvtk.to_vtk(a.actor.actor)
 
             if isinstance(a, vtk.vtkAssembly): #unpack assemblies
                 cl = vtk.vtkPropCollection()
@@ -716,7 +644,7 @@ def view(image=None,
                 tp.Update()
                 poly = tp.GetOutput()
                 prop = a.GetProperty()
-                if poly.GetNumberOfPolys():
+                if poly.GetNumberOfPolys() or poly.GetNumberOfStrips() or poly.GetNumberOfLines():
                     geometries.insert(0, poly)
                     geometry_colors.insert(0, prop.GetColor())
                     geometry_opacities.insert(0, prop.GetOpacity())
